import java.awt.Color;
import java.util.ArrayList;

public class DropGame
{
  private Environment theEnv;  // contains Piece objects

  public DropGame(Environment env)
  { theEnv = env; }

  /*** Part (a) ***/

  // returns null if no empty locations in column;
  // otherwise, returns the empty location with the
  // largest row index within the specified column;
  // precondition: 0 <= column < theEnv.numCols()
  public Location dropLocationForColumn(int column)
  {
    int maxRow = -1;

    for (int row = 0; row < theEnv.numRows(); row++)
      if (theEnv.isEmpty(new Location(row, column)))
        maxRow = row;

    if (maxRow < 0)
      return null;

    return new Location(maxRow, column);
  }

  /*** Part (b) ***/

  // returns true if dropping a piece of the given color into the
  // specified column matches color with three neighbors;
  // otherwise, returns false
  // precondition: 0 <= column < theEnv.numCols()
  public boolean dropMatchesNeighbors(int column, Color pieceColor)
  {
    Location dropLoc = dropLocationForColumn(column);
    if (dropLoc == null)
      return false;
    ArrayList<Location> nbrs = theEnv.neighborsOf(dropLoc);

    int count = 0;

    for (int index = 0; index < nbrs.size(); index++)
    {
      Location loc = nbrs.get(index);
      Piece piece = (Piece)theEnv.objectAt(loc);
      if (piece != null && piece.color().equals(pieceColor))
        count++;
    }

    return count == 3;
  }
}