import java.util.List;
import java.util.LinkedList;

public class PathFinder
{
  private Environment theEnv;

  public PathFinder(Environment env)
  { theEnv = env; }

  /*** Part (a) ***/

  // returns true if the specified locations are possible
  // starting and ending locations for an NE-path
  // otherwise, returns false
  private boolean possibleEnds(Location start, Location end)
  {
    return theEnv.isEmpty(start) && theEnv.isEmpty(end) &&
            start.row() >= end.row() && start.col() <= end.col();
  }

  /*** Part (b) ***/

  // returns a list containing a sequence of empty locations
  // that form an NE-path from start to end;
  // returns null if no such path exists
  // postcondition:  theEnv is unchanged
  public List<Location>  findNEPath(Location start, Location end)
  {
    if (!possibleEnds(start, end))
      return null;

    List<Location> path;

    if (start.equals(end))
    {
      path = new LinkedList<Location>();
      path.add(start);
      return path;
    }

    path = findNEPath(theEnv.getNeighbor(start, Direction.NORTH), end);
    if (path == null)
      path = findNEPath(theEnv.getNeighbor(start, Direction.EAST), end);

    if (path != null)
      path.add(0, start);

    return path;
  }
}





